package icfp2001;

import java.util.*;
import java.io.*;

class Main {

/**
 * Main Program
 */
public static void main (String args[]) {
   long tlimit = 0;

   if(args.length<2){
      System.out.println("usage:\nrunme <TIMELIMIT> \"<SML-Document>\"");
      System.exit(1);
   }
   tlimit = Integer.parseInt(args[0]);

   StringBuffer sb = new StringBuffer(10000);
   try {
      String line;
      BufferedReader f;
      f = new BufferedReader(
          new FileReader(args[1]));
      while ((line = f.readLine()) != null)
      {
         sb.append(line);
         sb.append("\n");
      }
      f.close();
   }
   catch (FileNotFoundException e) {
      for (int i=1; i<args.length; i++)
         sb.append(args[i]);
   }
   catch (IOException e) {
      for (int i=1; i<args.length; i++)
         sb.append(args[i]);
   }

   startOptimation(sb,tlimit);
}//main


/**
 * Starts the optimation of a document.
 */
public static void startOptimation(StringBuffer sb, long tlimit) {
   Date date = new Date();
   long secsStart = date.getTime();
   long secsNow   = 0;
   long maxTime   = (tlimit * 1000) - 1000;
   String s = sb.toString();

   while (true) {

      date = new Date();
      secsNow = date.getTime();
      if ((secsNow-secsStart) >= maxTime) break;
      OverlapInversion overlapInversion = new OverlapInversion();
      overlapInversion.startMe(s);
      while (overlapInversion.isAlive())  {
         date = new Date();
         secsNow = date.getTime();
         if ((secsNow-secsStart) >= maxTime) overlapInversion.breakIt();
      }
      s = overlapInversion.getString();

      date = new Date();
      secsNow = date.getTime();
      if ((secsNow-secsStart) >= maxTime) break;
      RedundancyElimination redundancyElimination = new RedundancyElimination();
      redundancyElimination.startMe(s);
      while (redundancyElimination.isAlive()) {
         date = new Date();
         secsNow = date.getTime();
         if ((secsNow-secsStart) >= maxTime) redundancyElimination.breakIt();
      }
      s = redundancyElimination.getString();

      date = new Date();
      secsNow = date.getTime();
      if ((secsNow-secsStart) >= maxTime) break;
      delTagSpace d = new delTagSpace();
      d.startMe(s);
      while (d.isAlive()) {
         date = new Date();
         secsNow = date.getTime();
         if ((secsNow-secsStart) >= maxTime) d.breakIt();
      }
      s = d.getString();

      date = new Date();
      secsNow = date.getTime();
      if ((secsNow-secsStart) >= maxTime) break;
      WhitespaceSimplification whitespaceSimplification = new WhitespaceSimplification();
      whitespaceSimplification.startMe(s);
      while (whitespaceSimplification.isAlive()) {
         date = new Date();
         secsNow = date.getTime();
         if ((secsNow-secsStart) >= maxTime) whitespaceSimplification.breakIt();
      }
      s = whitespaceSimplification.getString();


      date = new Date();
      secsNow = date.getTime();
      if ((secsNow-secsStart) >= maxTime) break;
      PLShortcut plShortcut = new PLShortcut();
      plShortcut.startMe(s);
      while (plShortcut.isAlive()) {
         date = new Date();
         secsNow = date.getTime();
         if ((secsNow-secsStart) >= maxTime) plShortcut.breakIt();
      }
      s = plShortcut.getString();

      date = new Date();
      secsNow = date.getTime();
      if ((secsNow-secsStart) >= maxTime) break;
      DelUselessSize delUselessSize = new DelUselessSize();
      delUselessSize.startMe(s);
      while (delUselessSize.isAlive()) {
         date = new Date();
         secsNow = date.getTime();
         if ((secsNow-secsStart) >= maxTime) delUselessSize.breakIt();
      }
      s = delUselessSize.getString();

      date = new Date();
      secsNow = date.getTime();
      if ((secsNow-secsStart) >= maxTime) break;
      ColorNesting colorNesting = new ColorNesting();
      colorNesting.startMe(s);
      while (colorNesting.isAlive()) {
         date = new Date();
         secsNow = date.getTime();
         if ((secsNow-secsStart) >= maxTime) colorNesting.breakIt();
      }
      s = colorNesting.getString();

      date = new Date();
      secsNow = date.getTime();
      if ((secsNow-secsStart) >= maxTime) break;
      deleteUseless delUse = new deleteUseless();
      delUse.startMe(s);
      while (delUse.isAlive()) {
         date = new Date();
         secsNow = date.getTime();
         if ((secsNow-secsStart) >= maxTime) delUse.breakIt();
      }
      s = delUse.getString();

      date = new Date();
      secsNow = date.getTime();
      if ((secsNow-secsStart) >= maxTime) break;
      EMElimination emElimination = new EMElimination();
      emElimination.startMe(s);
      while (emElimination.isAlive()) {
         date = new Date();
         secsNow = date.getTime();
         if ((secsNow-secsStart) >= maxTime) emElimination.breakIt();
      }
      s = emElimination.getString();

      date = new Date();
      secsNow = date.getTime();
      if ((secsNow-secsStart) >= maxTime) break;
      Underline underline = new Underline();
      underline.startMe(s);
      while (underline.isAlive()) {
         date = new Date();
         secsNow = date.getTime();
         if ((secsNow-secsStart) >= maxTime) underline.breakIt();
      }
      s = underline.getString();

      date = new Date();
      secsNow = date.getTime();
      if ((secsNow-secsStart) >= maxTime) break;
      WhitespaceCompession whitespaceCompession = new WhitespaceCompession();
      whitespaceCompession.startMe(s);
      while (whitespaceCompession.isAlive()) {
         date = new Date();
         secsNow = date.getTime();
         if ((secsNow-secsStart) >= maxTime) whitespaceCompession.breakIt();
      }
      s = whitespaceCompession.getString();

      break;
   }

   System.out.println(s);
}

}//class Main
